package cyano.electricadvantage.entities;

import cyano.electricadvantage.init.Power;
import cyano.electricadvantage.machines.HydroelectricGeneratorTileEntity;
import net.minecraft.block.Block;
import net.minecraft.block.BlockLiquid;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;

import java.util.List;

public class HydroturbineEntity extends net.minecraft.entity.Entity{

	public static final float DEGREES_PER_TICK = 30;
	private static final float RADIANS_TO_DEGREES = (float)(180 / Math.PI);
	private static final float DEGREES_TO_RADIANS = (float)(Math.PI / 180);
	public static final float PROPELLER_DAMAGE = 5f;
	public float rotation = 0;
	public boolean isSpinning = false;
	public TileEntity parent = null;
	private BlockPos parentCoord = null;
	private final long roffset;
	
	public HydroturbineEntity(World w) {
		super(w);
		this.field_70130_N = 0.9375F;
		this.field_70131_O = 0.9375F;
		this.func_70105_a(this.field_70130_N, this.field_70131_O);
		this.field_70156_m = true;
		roffset = w.field_73012_v.nextInt(8);
	}
	public HydroturbineEntity(World w, BlockPos parentTileEntity) {
		this(w,w.func_175625_s(parentTileEntity));
	}

	public HydroturbineEntity(World w, TileEntity parentTileEntity) {
		this(w);
		this.parent = parentTileEntity;
		BlockPos p = parentTileEntity.func_174877_v();
		this.func_70107_b(p.func_177958_n()+0.5, p.func_177956_o()-1, p.func_177952_p()+0.5);
	}

	@Override
	protected void func_70088_a() {
		// do nothing
	}

	@Override
	public void func_70071_h_(){
		super.func_70071_h_();
		IBlockState bs = this.field_70170_p.func_180495_p(func_180425_c());
		Block b = bs.func_177230_c();
		Vec3d directionVector;
		if(b instanceof BlockLiquid){
			directionVector = ((BlockLiquid)b).func_176197_a(func_130014_f_(), func_180425_c(), null, new Vec3d(0,0,0));
			isSpinning = !(directionVector.field_72450_a == 0 && directionVector.field_72449_c == 0);
		} else {
			isSpinning = false;
			directionVector = new Vec3d(0,0,0);
		}
		if(this.func_130014_f_().field_72995_K){
			// client-side only
			if(isSpinning){
				rotation += DEGREES_PER_TICK;
				this.field_70177_z = RADIANS_TO_DEGREES * (float)Math.atan2(-directionVector.field_72449_c, directionVector.field_72450_a) -90;
			} else {
				// do nothing
			}
		} else {
			// server-side only
			if(parent == null && parentCoord != null){
				parent = this.func_130014_f_().func_175625_s(parentCoord);
				parentCoord = null;
			}
			if(parent == null || parent.func_145837_r()){
				this.func_70076_C();
			} else if(parent instanceof HydroelectricGeneratorTileEntity){
				HydroelectricGeneratorTileEntity generator = (HydroelectricGeneratorTileEntity)parent;
				if(isSpinning){
					generator.addEnergy(HydroelectricGeneratorTileEntity.ENERGY_PER_TICK, Power.ELECTRIC_POWER);
				}
				generator.setActive(isSpinning);
			}
			if(isSpinning && func_130014_f_().func_82737_E() % 8 == roffset){
				// slice-n-dice unfortunate souls caught in the blades
				List<Entity> victims = func_130014_f_().func_72872_a(EntityLivingBase.class, this.func_174813_aQ());
				for(Entity e : victims){
					e.func_70097_a(Power.propeller_damage, PROPELLER_DAMAGE);
				}
			}
		}
	}
	
	@Override
	protected void func_70037_a(NBTTagCompound root) {
		if(root.func_74764_b("parentPos")){
			int[] coords = root.func_74759_k("parentPos");
			if(coords.length >= 3){
				parentCoord = new BlockPos(coords[0],coords[1],coords[2]);
			}
		}
	}

	@Override
	protected void func_70014_b(NBTTagCompound root) {
		if(parent != null){
			int[] coords = new int[3];
			BlockPos p = parent.func_174877_v();
			coords[0] = p.func_177958_n();
			coords[1] = p.func_177956_o();
			coords[2] = p.func_177952_p();
			root.func_74783_a("parentPos", coords);
		}
	}
	
}
