package cyano.electricadvantage.graphics;

import cyano.electricadvantage.ElectricAdvantage;
import cyano.electricadvantage.machines.ElectricDrillTileEntity;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.VertexBuffer;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT) // This is needed for classes that extend client-only classes
public class LaserDrillRenderer extends TileEntitySpecialRenderer{

	
	private final ResourceLocation texture = new ResourceLocation(ElectricAdvantage.MODID+":textures/materials/laser_beam.png");

	private static final float RADIANS_TO_DEGREES = (float)(180 / Math.PI);
	
	
	public LaserDrillRenderer() {
		super();
	}

	
	@Override
	public void func_180535_a(final TileEntity te, final double x, final double y, final double z, final float partialTick, int meta) {
		if(te instanceof ElectricDrillTileEntity){
			// partialTick is guaranteed to range from 0 to 1
			GlStateManager.func_179094_E();
			GlStateManager.func_179109_b((float)x, (float)y, (float)z);

			render((ElectricDrillTileEntity)te,te.func_145831_w(),te.func_174877_v(),partialTick);
			
			GlStateManager.func_179121_F();
		}
	}
	
	private void render(ElectricDrillTileEntity e, World world, BlockPos pos, float partialTick){
		float laserBlastLength = e.getDrillLength();

		if(laserBlastLength > 0){
			this.func_147499_a(texture);
			final Tessellator tessellator = Tessellator.func_178181_a();

			final VertexBuffer worldRenderer = tessellator.func_178180_c();


			final float laserU0 = 0;
			final float laserU1 = 0;
			final float laserV0 = 1.0f;
			final float laserV1 = 1.0f;

			final float radius = 0.25f;
			final float laserRadius = 0.125f;


			float tickRemainder = 1 - partialTick;
			EnumFacing dir = e.getFacing();

			float rotY = 0, rotX = 0;
			switch(dir){
			case NORTH:{
				// do nothing
				break;
			}
			case EAST:{
				rotY = -90;
				break;
			}
			case SOUTH:{
				rotY = 180;
				break;
			}
			case WEST:{
				rotY = 90;
				break;
			}
			case UP:{
				rotX = 90;
				break;
			}
			case DOWN:{
				rotX = -90;
				break;
			}
			default:{
				// do nothing
				break;
			}
			}
			
			GlStateManager.func_179140_f();
			
			GlStateManager.func_179109_b(0.5f, 0.5f, 0.5f);
			GlStateManager.func_179114_b(rotY, 0.0f, 1.0f, 0.0f);
			GlStateManager.func_179114_b(rotX, 1.0f, 0.0f, 0.0f);

			int lightValue = 15 << 20 | 15 << 4;
			int lmapX = lightValue >> 16 & 0xFFFF;
			int lmapY = lightValue & 0xFFFF;
			worldRenderer.func_181668_a(7, DefaultVertexFormats.field_181711_k);

			final double x1, y1, z1, x2, y2, z2,  x3, y3, z3, x4, y4, z4;
			x1 = 0;
			y1 = -laserRadius;
			z1 = 0;
			x2 = 0;
			y2 = laserRadius;
			z2 = -laserBlastLength;

			x3 = -laserRadius;
			y3 = 0;
			z3 = z1;
			x4 = laserRadius;
			y4 = 0;
			z4 = z2;

			worldRenderer.func_181662_b( x1, y1, z1).func_187315_a(laserU0, laserV1).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x2, y1, z2).func_187315_a(laserU1, laserV1).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x2, y2, z2).func_187315_a(laserU1, laserV0).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x1, y2, z1).func_187315_a(laserU0, laserV0).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();

			worldRenderer.func_181662_b( x1, y2, z1).func_187315_a(laserU0, laserV1).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x2, y2, z2).func_187315_a(laserU1, laserV1).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x2, y1, z2).func_187315_a(laserU1, laserV0).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x1, y1, z1).func_187315_a(laserU0, laserV0).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();


			worldRenderer.func_181662_b( x3, y3, z3).func_187315_a(laserU0, laserV1).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x3, y4, z4).func_187315_a(laserU1, laserV1).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x4, y4, z4).func_187315_a(laserU1, laserV0).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x4, y3, z3).func_187315_a(laserU0, laserV0).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();

			worldRenderer.func_181662_b( x4, y3, z3).func_187315_a(laserU0, laserV1).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x4, y4, z4).func_187315_a(laserU1, laserV1).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x3, y4, z4).func_187315_a(laserU1, laserV0).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();
			worldRenderer.func_181662_b( x3, y3, z3).func_187315_a(laserU0, laserV0).func_187314_a(lmapX, lmapY).func_181666_a(1f, 1f, 1f, 1f).func_181675_d();

			tessellator.func_78381_a();
			GlStateManager.func_179145_e();

		}
	}

	
}
