package cyano.electricadvantage.items;

import cyano.electricadvantage.common.IRechargeableItem;
import cyano.electricadvantage.init.Power;
import cyano.poweradvantage.api.ConduitType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.text.translation.I18n;

import java.text.NumberFormat;
import java.util.List;


public class BatteryItem extends net.minecraft.item.Item implements IRechargeableItem {
	private final float capacity;
	private final NumberFormat nf;
	
	public BatteryItem(float capacity){
		this.capacity = capacity;
		nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(0);
		nf.setMinimumFractionDigits(0);
		super.func_77625_d(1);
	}
	
	@Override
	public float getEnergy(ItemStack stack) {
		if(stack.func_77973_b() instanceof IRechargeableItem){
			BatteryItem battery = (BatteryItem)stack.func_77973_b();
			NBTTagCompound dataTag;
			if(!stack.func_77942_o()){
				return battery.capacity; // new battery starts with 100% charge
			} else {
				dataTag = stack.func_77978_p();
				if(dataTag.func_74764_b("energy")){
					return dataTag.func_74760_g("energy");
				} else {
					return battery.capacity; // new battery starts with 100% charge
				}
			}
		} else {
			return 0;
		}
	}
	@Override
	public void setEnergy(ItemStack stack, float energy){
		if(stack.func_77973_b() instanceof BatteryItem){
			NBTTagCompound dataTag;
			if(!stack.func_77942_o()){
				dataTag = new NBTTagCompound();
			} else {
				dataTag = stack.func_77978_p();
			}
			dataTag.func_74776_a("energy", energy);
			stack.func_77982_d(dataTag);
		}
	}
	/**
	 * Adds energy to a battery item
	 * @param stack battery item stack
	 * @param energy energy to add
	 * @return Returns the actual amount of energy added
	 */
	@Override
	public float addEnergy(ItemStack stack, float energy, ConduitType type){
		if(ConduitType.areSameType(type, Power.ELECTRIC_POWER) && stack.func_77973_b() instanceof BatteryItem){
			BatteryItem battery = (BatteryItem)stack.func_77973_b();
			NBTTagCompound dataTag;
			if(!stack.func_77942_o()){
				dataTag = new NBTTagCompound();
			} else {
				dataTag = stack.func_77978_p();
			}
			float currentEnergy;
			if(dataTag.func_74764_b("energy")){
				currentEnergy = dataTag.func_74760_g("energy");
			} else {
				currentEnergy = battery.capacity;
			}
			float newEnergy = Math.max(0, Math.min(currentEnergy+energy, battery.capacity));
			dataTag.func_74776_a("energy", newEnergy);
			stack.func_77982_d(dataTag);
			return newEnergy - currentEnergy;
		} else {
			return 0;
		}
	}
	
	
	@Override
	public float getMaxEnergy(ItemStack item){
		return this.capacity;
	}
	
	@Override
	public ConduitType getEnergyType(){
		return Power.ELECTRIC_POWER;
	}

	@Override
	public void func_77624_a(ItemStack stack, EntityPlayer player, List list, boolean b){
		super.func_77624_a(stack,player,list,b);
		StringBuilder sb = new StringBuilder();
		float max = getMaxEnergy(stack);
		if(max <= 0) return; // not a battery!
		float e = getEnergy(stack);
		//sb.append(nf.format(e)).append('/').append(nf.format(max)).append(" kJ");
		sb.append(nf.format(100*e/max)).append("% ").append(I18n.func_74838_a("tooltip.battery.charge"));
		list.add(sb.toString());
		StringBuilder sb2 = new StringBuilder();
		sb2.append('(').append(nf.format(max)).append("kJ ").append(I18n.func_74838_a("tooltip.battery.capacity")).append(')');
		list.add(sb2.toString());
	}
}
