package cyano.electricadvantage.machines;

import cyano.electricadvantage.init.Power;
import cyano.poweradvantage.api.*;
import cyano.poweradvantage.conduitnetwork.ConduitRegistry;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.Explosion;
import net.minecraft.world.World;

import java.util.Random;

public class ElectricDrillBlock extends GUIBlock implements ITypedConduit {



	private final ConduitType type;
	private final ConduitType[] types = new ConduitType[1];

	/**
	 * Blockstate property
	 */
	public static final PropertyDirection FACING = PropertyDirection.func_177714_a("facing");
	/**
	 * Blockstate property
	 */
	public static final PropertyBool ACTIVE = PropertyBool.func_177716_a("active");


	public ElectricDrillBlock(){
		super(Material.field_76233_E);
		this.type = Power.ELECTRIC_POWER;
		types[0] = this.type;
		super.func_149711_c(0.75f);
		this.func_180632_j(func_176223_P().func_177226_a(ACTIVE, false).func_177226_a(FACING, EnumFacing.DOWN));
	}


	/**
	 * Creates a blockstate
	 */
	@Override
	protected BlockStateContainer func_180661_e() {
		return new BlockStateContainer(this, new IProperty[] { ACTIVE,FACING });
	}

	@Override
	public ElectricMachineTileEntity func_149915_a(World w, int m) {
		return new ElectricDrillTileEntity();
	}

	

	/**
	 * Override of default block behavior
	 */
	@Override
	public void func_176213_c(final World world, final BlockPos coord, final IBlockState state) {
		ConduitRegistry.getInstance().conduitBlockPlacedEvent(world, world.field_73011_w.getDimension(), coord, type);
	}
	/**
	 * This method is called when the block is removed from the world by an entity.
	 */
	@Override
	public void func_176206_d(World w, BlockPos coord, IBlockState state){
		super.func_176206_d(w, coord, state);
		ConduitRegistry.getInstance().conduitBlockRemovedEvent(w, w.field_73011_w.getDimension(), coord, type);
	}
	/**
	 * This method is called when the block is destroyed by an explosion.
	 */
	@Override
	public void func_180652_a(World w, BlockPos coord, Explosion boom){
		super.func_180652_a(w, coord, boom);
		ConduitRegistry.getInstance().conduitBlockRemovedEvent(w, w.field_73011_w.getDimension(), coord, type);
	}

	/**
	 * Used to decides whether or not a conduit should connect to this block 
	 * based on its energy type.
	 * @return The type of energy for this block 
	 */
	@Override
	public ConduitType[] getTypes() {
		return types;
	}

	@Override
	public boolean isPowerSink(ConduitType conduitType) {
		return true;
	}

	@Override
	public boolean isPowerSource(ConduitType conduitType) {
		return false;
	}


	@Override
	public boolean canAcceptConnection(PowerConnectorContext connection) {
		return ConduitType.areSameType( type, connection.powerType);
	}


	/**
	 * Override of default block behavior
	 */
	@Override
	public Item func_180660_a(final IBlockState state, final Random prng, final int i3) {
		return Item.func_150898_a(this);
	}



	/**
	 * Creates the blockstate of this block when it is placed in the world
	 */
	@Override
	public IBlockState func_180642_a(final World world, final BlockPos coord, final EnumFacing facing, 
			final float f1, final float f2, final float f3, 
			final int meta, final EntityLivingBase player) {
		return this.func_176223_P().func_177226_a( FACING, facing.func_176734_d());
	}

	/**
	 * Creates the blockstate of this block when it is placed in the world
	 */
	@Override
	public void func_180633_a(final World world, final BlockPos coord, final IBlockState bs, 
			final EntityLivingBase placer, final ItemStack srcItemStack) {
		if (srcItemStack.func_82837_s()) {
			final TileEntity tileEntity = world.func_175625_s(coord);
			if (tileEntity instanceof PoweredEntity){
				((PoweredEntity)tileEntity).setCustomInventoryName(srcItemStack.func_82833_r());
			}
		}
	}




	/**
	 * Sets the default blockstate
	 * @param w World instance
	 * @param coord Block coordinate
	 * @param state Block state
	 */
	protected void setDefaultFacing(final World w, final BlockPos coord, final IBlockState state) {
		if (w.field_72995_K) {
			return;
		}
		EnumFacing enumFacing = EnumFacing.DOWN;
		w.func_180501_a(coord, state.func_177226_a((IProperty) FACING, (Comparable)enumFacing), 2);
	}


	/**
	 * Converts metadata into blockstate
	 */
	@Override
	public IBlockState func_176203_a(final int metaValue) {
		EnumFacing enumFacing = metaToFacing(metaValue);
		if (enumFacing.func_176740_k() == EnumFacing.Axis.Y) {
			enumFacing = EnumFacing.NORTH;
		}
		return this.func_176223_P().func_177226_a( FACING, enumFacing)
				.func_177226_a(ACTIVE, (metaValue & 0x8) != 0);
	}
	
	/**
	 * Converts blockstate into metadata
	 */
	@Override
	public int func_176201_c(final IBlockState bs) {
		int extraBit;
		if((Boolean)(bs.func_177229_b(ACTIVE))){
			extraBit = 0x8;
		} else {
			extraBit = 0;
		}
		return facingToMeta((EnumFacing)bs.func_177229_b( FACING)) | extraBit;
	}
	
	private int facingToMeta(EnumFacing f){
		return f.func_176745_a();
	}
	private EnumFacing metaToFacing(int i){
		int f = i & 0x07;
		return EnumFacing.values()[f];
	}


	@Override
	public int func_180641_l(IBlockState bs, World w, BlockPos p) {
		TileEntity te = w.func_175625_s(p);
		if(te instanceof ElectricMachineTileEntity){
			return ((ElectricMachineTileEntity)te).getComparatorOutput();
		}
		return 0;

	}

	@Override
	public boolean func_149740_M(IBlockState bs) {
		return true;
	}

	@Override
	public void func_180663_b(World world, BlockPos pos, IBlockState state){
		TileEntity te = world.func_175625_s(pos);
		if (te instanceof IInventory)
		{
			InventoryHelper.func_180175_a(world, pos, (IInventory)te);
			((IInventory)te).func_174888_l();
			world.func_175666_e(pos, this);
		}
		super.func_180663_b(world, pos, state);
	}


	



}
