package cyano.electricadvantage.machines;

import cyano.electricadvantage.init.Blocks;
import cyano.electricadvantage.init.Power;
import cyano.poweradvantage.util.InventoryWrapper;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.SoundEvents;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ElectricDrillTileEntity extends ElectricMachineTileEntity{


	public static final int MAX_RANGE = 64;
	public static final float ENERGY_COST_MOVE = 250f;
	public static final float ENERGY_COST_PROGRESS_TICK = 24f;
	public static final float MINING_TIME_FACTOR = 32.0f;

	
	private final int[] dataSyncArray = new int[4];
	private int progress = 0;
	private int progressGoal = 0;
	private BlockPos targetBlockCoord = null;
	private Block[] targetBlocks = null;
	private List<ItemStack> targetBlockItems = null;
	private int laserLength = 0;

	
	private boolean deferred = false;
	
	public ElectricDrillTileEntity() {
		super(ElectricDrillTileEntity.class.getSimpleName(), 
				0, 6, 0);
	}

	@Override
	public boolean isPowered() {
		return getEnergy() > ENERGY_COST_PROGRESS_TICK;
	}

	private float[] progArr = new float[1];
	@Override
	public float[] getProgress() {
		if(progressGoal > 0){
			progArr[0] = (float)progress / (float)progressGoal;
		} else {
			progArr[0] = 0;
		}
		return progArr;
	}
	@Override
	public void tickUpdate(boolean isServerWorld) {
		if(isServerWorld){
			if(deferred){
				targetBlock(targetBlockCoord);
			}
			updateFacing();
			
			// disabled by redstone
			if(hasRedstoneSignal()){
				if (progress > 0){
					progress = 0;
				}
			} else {
				if(targetBlockCoord != null){
					// mining time
					if(getEnergy() > ENERGY_COST_PROGRESS_TICK && hasSpaceForItems(targetBlockItems) && canMine(targetBlockCoord)){
						this.subtractEnergy(ENERGY_COST_PROGRESS_TICK, Power.ELECTRIC_POWER);
						progress++;
						if(progress >= progressGoal){
							// Mined it!
							playSoundEffect(targetBlockCoord.func_177958_n()+0.5, targetBlockCoord.func_177956_o()+0.5, targetBlockCoord.func_177952_p()+0.5, targetBlocks[0].func_185467_w().func_185841_e(), 0.5f, 1f);
							playSoundEffect(func_174877_v().func_177958_n()+0.5, func_174877_v().func_177956_o()+0.5, func_174877_v().func_177952_p()+0.5, SoundEvents.field_187747_eB, 0.5f, 1f);
							BlockPos[] targets = this.getArea(targetBlockCoord);
							for(int i = 0; i < targets.length; i++)
								if(canMine(targets[i])){
									func_145831_w().func_175698_g(targets[i]);
								}
							for(ItemStack item : targetBlockItems){
								addItem(item);
							}
							untargetBlock();
						}
					} else {
						// lose progress
						if(progress > 0)progress--;
					}
				}
				
			}

			this.setActiveState(isActive());
		}
		
	}
	
	private boolean hasSpaceForItems(List<ItemStack> items) {
		if(items.size() == 1){
			return super.hasSpaceForItemInOutputSlots(items.get(0));
		} else {
			for(ItemStack i : items){
				if(super.hasSpaceForItemInOutputSlots(i) == false){
					return false;
				}
			}
			return true;
		}
	}

	public ItemStack addItem(ItemStack in){
		return super.insertItemToOutputSlots(in);
	}

	private EnumFacing trackDirection(){
		for(EnumFacing dir : EnumFacing.values()){
			if(func_145831_w().func_180495_p(func_174877_v().func_177972_a(dir)).func_177230_c() == Blocks.electric_track){
				return dir;
			}
		}
		return null;
	}
	
	private boolean followTrack(){
		if(this.getEnergy() < ENERGY_COST_MOVE) return false;
		EnumFacing trackDir = trackDirection();
		if(trackDir == null) return false;
		
		// clone this block into neighboring block
		this.untargetBlock();
		World w = func_145831_w();
		BlockPos nextPos = func_174877_v().func_177972_a(trackDir);
		w.func_180501_a(nextPos, w.func_180495_p(func_174877_v()), 2);
		ElectricDrillTileEntity te = (ElectricDrillTileEntity)w.func_175625_s(nextPos);
		NBTTagCompound dataTransfer = new NBTTagCompound();
		this.func_189515_b(dataTransfer);
		te.func_145839_a(dataTransfer);
		te.func_174878_a(nextPos);
		te.func_145829_t();
		te.func_70296_d();
		Arrays.fill(this.getInventory(), null);
		
		// replace this block with steam pipe
		w.func_175656_a(func_174877_v().func_177972_a(getFacing()), cyano.poweradvantage.init.Blocks.steel_frame.func_176223_P());
		w.func_180501_a(func_174877_v(), Blocks.electric_conduit.func_176223_P(), 2);
		
		
		this.subtractEnergy(ENERGY_COST_MOVE, getType());
		return true;
	}
	

	@Override
	public int[] getDataFieldArray() {
		return dataSyncArray;
	}

	@Override
	public void prepareDataFieldsForSync() {
		dataSyncArray[0] = Float.floatToRawIntBits(this.getEnergy());
		dataSyncArray[1] = progress;
		dataSyncArray[2] = progressGoal;
		dataSyncArray[3] = laserLength;
	}

	@Override
	public void onDataFieldUpdate() {
		this.setEnergy(Float.intBitsToFloat(dataSyncArray[0]), this.getType());
		this.progress = dataSyncArray[1];
		progressGoal = dataSyncArray[2];
		laserLength = dataSyncArray[3];
		updateFacing();
	}
	private float oldEnergy = 0;
	private int oldProgress = 0;
	private int oldLength = 0;
	@Override public void powerUpdate(){
		if(deferred){
			targetBlock(targetBlockCoord);
		}
		super.powerUpdate();
		boolean flagSync = progress != oldProgress || oldEnergy != getEnergy();
		oldProgress = progress;
		oldEnergy = getEnergy();
		
		boolean redstone = hasRedstoneSignal();
		
		
		EnumFacing f = getFacing();
		BlockPos n = func_174877_v().func_177972_a(f);
		
		// manage drill bits and find next block
		if(redstone || this.getEnergy() <= 0){
			// no power
			flagSync = true;
			this.untargetBlock();
		} else {
			// drill baby drill!
			if(targetBlockCoord == null ){
				// find new block
				boolean hitEnd = false;
				for(int i = 0; i <= MAX_RANGE ; i++){
					if(i == MAX_RANGE){
						hitEnd = true;
						break;
					}
				
					if(func_145831_w().func_175623_d(n) || func_145831_w().func_180495_p(n).func_177230_c().func_176200_f(func_145831_w(), n)){
						// this block is not worth mining, replace it
						n = n.func_177972_a(f);
						if(n.func_177956_o() == 0 || n.func_177956_o() == 255){
							hitEnd = true;
							break;
						}
					} else {
						// found a block!
						if(canMine(n)){
							targetBlock(n);
						} else {
							hitEnd = true;
						}
						flagSync = true;
						break;
					}
				}
				// if hit end of range, move along track
				if(hitEnd){
					boolean moved = followTrack();
					if(moved){
						cyano.poweradvantage.conduitnetwork.ConduitRegistry.getInstance()
								.conduitBlockRemovedEvent(func_145831_w(), func_145831_w().field_73011_w.getDimension(), func_174877_v(), getType());
						return;
					}
				}
			} else {
				// currently drilling a block
				// block revalidation
				BlockPos[] targets = getArea(targetBlockCoord);
				for(int i = 0; i < targets.length; i++){
					if(!func_145831_w().func_180495_p(targets[i]).func_177230_c().equals(targetBlocks[i])){
						// Block changed! Invalidate!
						untargetBlock();
						flagSync = true;
						break;
					}
				}
			}
		}
		
		calculateDrillLength();
		if(oldLength != laserLength){
			oldLength = laserLength;
			flagSync = true;
		}
		
		if(flagSync){
			this.sync();
		}
		

		// push inventory to adjacent chest
		BlockPos adj = func_174877_v().func_177972_a(f.func_176734_d());
		if(!redstone && !func_145831_w().func_175623_d(adj)){
			inventoryTransfer(adj,f);
		}
		
		// fry any idiots who stand in the laser's path
		if(isActive() && targetBlocks != null && targetBlockCoord != null){
			BlockPos pos = func_174877_v();
			AxisAlignedBB beamArea = new AxisAlignedBB(targetBlockCoord.func_177958_n(),targetBlockCoord.func_177956_o(),targetBlockCoord.func_177952_p(),
					pos.func_177958_n()+1,pos.func_177956_o()+1,pos.func_177952_p()+1);
			List<EntityLivingBase> victims = func_145831_w().func_72872_a(EntityLivingBase.class, beamArea);
			for(Entity e : victims){
				e.func_70015_d(3);
				e.func_70097_a(Power.laser_damage, 2f);
			}
		}
	}

	
	public int getDrillLength(){
		return laserLength;
	}
	
	public void calculateDrillLength(){
		if(this.isActive() && this.targetBlocks != null && this.targetBlockCoord != null){
			BlockPos pos = func_174877_v();
			// distance calculation is taking a short-cut by assuming that 2 out of the 3 XYZ coordinates are identical
			laserLength = MathHelper.func_76130_a((pos.func_177958_n() - targetBlockCoord.func_177958_n()) + (pos.func_177956_o() - targetBlockCoord.func_177956_o()) + (pos.func_177952_p() - targetBlockCoord.func_177952_p()));
		} else {
			laserLength = 0;
		}
	}

	private void targetBlock (BlockPos pos){
		BlockPos[] targets = getArea(pos);
		progress = 0;
		targetBlockCoord = pos;
		progressGoal = 0;
		targetBlocks = new Block[targets.length];
		targetBlockItems = new ArrayList<>(targets.length);
		for(int i = 0; i < targets.length; i++){
			targetBlocks[i] = func_145831_w().func_180495_p(targets[i]).func_177230_c();
			progressGoal += this.getBlockStrength(targets[i]);
			targetBlockItems.addAll(targetBlocks[i].getDrops(func_145831_w(), targets[i], func_145831_w().func_180495_p(targets[i]), 0));
		}
		deferred = false;
	}
	
	private BlockPos[] getArea(BlockPos center){
		BlockPos[] a = new BlockPos[9];
		a[0] = center;
		switch(this.getFacing().func_176740_k()){
			case X: 
				a[1] = center.func_177978_c();
				a[2] = center.func_177977_b();
				a[3] = center.func_177968_d();
				a[4] = center.func_177984_a();
				a[5] = a[2].func_177978_c();
				a[6] = a[2].func_177968_d();
				a[7] = a[4].func_177978_c();
				a[8] = a[4].func_177968_d();
				break;
			case Z:
				a[1] = center.func_177977_b();
				a[2] = center.func_177976_e();
				a[3] = center.func_177984_a();
				a[4] = center.func_177974_f();
				a[5] = a[1].func_177974_f();
				a[6] = a[1].func_177976_e();
				a[7] = a[3].func_177974_f();
				a[8] = a[3].func_177976_e();
				break;
			case Y:
			default:
				a[1] = center.func_177978_c();
				a[2] = center.func_177976_e();
				a[3] = center.func_177968_d();
				a[4] = center.func_177974_f();
				a[5] = a[1].func_177974_f();
				a[6] = a[1].func_177976_e();
				a[7] = a[3].func_177974_f();
				a[8] = a[3].func_177976_e();
				break;
		}
		return a;
	}
	
	private void deferredTargetBlock (BlockPos n){
		targetBlockCoord = n;
		deferred = true;
	}
	
	private void untargetBlock(){
		progress = 0;
		progressGoal = 0;
		targetBlockCoord = null;
		targetBlocks = null;
		targetBlockItems = null;
	}
	
	private EnumFacing facingCache = null;
	public EnumFacing getFacing(){
		if(facingCache == null) updateFacing();
		return facingCache;
	}
	private void updateFacing(){
		final EnumFacing old = facingCache;
		facingCache = (EnumFacing)field_145850_b.func_180495_p(func_174877_v()).func_177229_b(ElectricDrillBlock.FACING);
		if(old != null && old != facingCache){
			untargetBlock();
			this.sync();
		}
	}
	
	
	private boolean canMine(BlockPos coord){
		IBlockState bs  = func_145831_w().func_180495_p(coord);
		Block b = bs.func_177230_c();
		// indestructable blocks have negative hardness
		return !(b.func_176195_g(bs, func_145831_w(), coord) < 0);
	}
	
	private int getBlockStrength(BlockPos coord){
		if(func_145831_w().func_175623_d(coord)){
			return 0;
		}
		IBlockState bs  = func_145831_w().func_180495_p(coord);
		Block block = bs.func_177230_c();
		return (int)(Math.max(MINING_TIME_FACTOR * block.func_176195_g(bs, func_145831_w(), coord),0.5f * MINING_TIME_FACTOR));
	}
	
	

	private void inventoryTransfer(BlockPos adj, EnumFacing otherFace) {
		ItemStack[] inventory = this.getInventory();
		TileEntity te = func_145831_w().func_175625_s(adj);
		if(te instanceof IInventory ){
			ISidedInventory inv = InventoryWrapper.wrap((IInventory)te);
			int[] accessibleSlots = inv.func_180463_a(otherFace);
			if(accessibleSlots.length == 0) return;
			for(int mySlot = 0; mySlot < inventory.length; mySlot++){
				if(inventory[mySlot] == null) continue;
				for(int i = 0; i < accessibleSlots.length; i++){
					int theirSlot = accessibleSlots[i];
					ItemStack theirItem = inv.func_70301_a(theirSlot);
					if(inv.func_180462_a(theirSlot, inventory[mySlot], otherFace)){
						if(theirItem == null){
							ItemStack newItem = inventory[mySlot].func_77946_l();
							newItem.field_77994_a = 1;
							inv.func_70299_a(theirSlot, newItem);
							inventory[mySlot].field_77994_a--;
							if(inventory[mySlot].field_77994_a <= 0) inventory[mySlot] = null;
							return;
						} else if(ItemStack.func_179545_c(theirItem, inventory[mySlot]) 
								&& ItemStack.func_77970_a(theirItem, inventory[mySlot])
								&& theirItem.field_77994_a < theirItem.func_77976_d()
								&& theirItem.field_77994_a < inv.func_70297_j_()){
							theirItem.field_77994_a++;
							inventory[mySlot].field_77994_a--;
							if(inventory[mySlot].field_77994_a <= 0) inventory[mySlot] = null;
							return;
						}
					}
				}
			}
			
		}
	}


	
	public float getProgressLevel(){
		if(progressGoal > 0) 
			return (float)progress / (float)progressGoal;
		return 0;
	}
	
	
	
	
	@Override
	protected void saveTo(NBTTagCompound tagRoot) {
		tagRoot.func_74777_a("progress",(short)progress);
		if(targetBlockCoord != null){
			tagRoot.func_74768_a("targetX", targetBlockCoord.func_177958_n());
			tagRoot.func_74768_a("targetY", targetBlockCoord.func_177956_o());
			tagRoot.func_74768_a("targetZ", targetBlockCoord.func_177952_p());
		}
	}

	@Override
	protected void loadFrom(NBTTagCompound tagRoot) {
		if(tagRoot.func_74764_b("progress")){
			progress = tagRoot.func_74765_d("progress");
		}
		if(tagRoot.func_74764_b("targetX") && tagRoot.func_74764_b("targetY") && tagRoot.func_74764_b("targetZ")){
			int x = tagRoot.func_74762_e("targetX");
			int y = tagRoot.func_74762_e("targetY");
			int z = tagRoot.func_74762_e("targetZ");
			// Note: world object for tile entities is set AFTER loading them from NBT
			if(func_145831_w() == null){
				this.deferredTargetBlock(new BlockPos(x,y,z));
			} else {
				this.targetBlock(new BlockPos(x,y,z));
			}
		}
	}

	@Override
	protected boolean isValidInputItem(ItemStack item) {
		return true;
	}
	
	@Override
	protected void setActiveState(boolean active){
		IBlockState oldState = func_145831_w().func_180495_p(func_174877_v());
		if(oldState.func_177230_c() instanceof ElectricDrillBlock 
				&& (Boolean)oldState.func_177229_b(ElectricDrillBlock.ACTIVE) != active ){
			final TileEntity save = this;
			final World w = func_145831_w();
			final BlockPos pos = this.func_174877_v();
			IBlockState newState = oldState.func_177226_a(ElectricDrillBlock.ACTIVE, active);
			w.func_180501_a(pos, newState,3);
			if(save != null){
				w.func_175713_t(pos);
				save.func_145829_t();
				w.func_175690_a(pos, save);
			}
		}
	}
	@Override
	protected void setPowerState(boolean powered){
		// do nothing, this block does not have a powered state
	}

	@Override
	public boolean isActive(){
		return isPowered() && (!this.hasRedstoneSignal() && (this.getEnergy() > 0 || (Boolean)func_145831_w().func_180495_p(func_174877_v()).func_177229_b(ElectricDrillBlock.ACTIVE)));
	}
	
	// Helps with laser rendering
	final private int renderRange = MAX_RANGE * 2;
	@SideOnly(Side.CLIENT)
	@Override
	public AxisAlignedBB getRenderBoundingBox()
	{
		return new AxisAlignedBB(func_174877_v().func_177982_a(-renderRange, -renderRange, -renderRange), func_174877_v().func_177982_a(renderRange, renderRange, renderRange));
		//return super.INFINITE_EXTENT_AABB;
	}
}
