package cyano.electricadvantage.machines;

import cyano.electricadvantage.init.Power;
import cyano.poweradvantage.api.ConduitType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

public abstract class ElectricMachineBlock extends cyano.poweradvantage.api.simple.BlockSimplePowerMachine{

	/**
	 * Blockstate property
	 */
	public static final PropertyBool ACTIVE = PropertyBool.func_177716_a("active");
	/**
	 * Blockstate property
	 */
	public static final PropertyBool POWERED = PropertyBool.func_177716_a("powered");

	public ElectricMachineBlock() {
		this(Material.field_76233_E);
	}

	public ElectricMachineBlock(Material m, ConduitType type) {
		super(m, 0.75f, type);
		this.func_180632_j(func_176223_P().func_177226_a(ACTIVE, false).func_177226_a(POWERED, false).func_177226_a(FACING, EnumFacing.NORTH));
	}

	public ElectricMachineBlock(Material m) {
		this(m,Power.ELECTRIC_POWER);
	}
	
	/**
	 * Creates a blockstate
	 */
	@Override
	protected BlockStateContainer func_180661_e() {
		return new BlockStateContainer(this, new IProperty[] { ACTIVE,FACING,POWERED });
	}

	/**
	 * Converts metadata into blockstate
	 */
	@Override
	public IBlockState func_176203_a(final int metaValue) {
		EnumFacing enumFacing = metaToFacing(metaValue);
		if (enumFacing.func_176740_k() == EnumFacing.Axis.Y) {
			enumFacing = EnumFacing.NORTH;
		}
		return this.func_176223_P().func_177226_a( FACING, enumFacing)
				.func_177226_a(ACTIVE, (metaValue & 0x4) != 0)
				.func_177226_a(POWERED, (metaValue & 0x8) != 0);
	}
	
	/**
	 * Converts blockstate into metadata
	 */
	@Override
	public int func_176201_c(final IBlockState bs) {
		int extraBit;
		if((Boolean)(bs.func_177229_b(ACTIVE))){
			extraBit = 0x4;
		} else {
			extraBit = 0;
		}
		if((Boolean)(bs.func_177229_b(POWERED))){
			extraBit = extraBit | 0x8;
		}
		return facingToMeta((EnumFacing)bs.func_177229_b( FACING)) | extraBit;
	}
	
	private int facingToMeta(EnumFacing f){
		switch(f){
			case NORTH: return 0;
			case WEST: return 1;
			case SOUTH: return 2;
			case EAST: return 3;
			default: return 0;
		}
	}
	private EnumFacing metaToFacing(int i){
		int f = i & 0x03;
		switch(f){
			case 0: return EnumFacing.NORTH;
			case 1: return EnumFacing.WEST;
			case 2: return EnumFacing.SOUTH;
			case 3: return EnumFacing.EAST;
			default: return EnumFacing.NORTH;
		}
	}
	
	@Override
	public abstract ElectricMachineTileEntity func_149915_a(World w, int m);

	@Override
	public int func_180641_l(IBlockState bs, World w, BlockPos p){
		return getComparatorInputOverride(w,p);
	}
	public int getComparatorInputOverride(World w, BlockPos p) {
		TileEntity te = w.func_175625_s(p);
		if(te instanceof ElectricMachineTileEntity){
			return ((ElectricMachineTileEntity)te).getComparatorOutput();
		}
		return 0;
		
	}

	@Override
	public boolean func_149740_M(IBlockState bs){
		return hasComparatorInputOverride();
	}
	public boolean hasComparatorInputOverride() {
		return true;
	}
	
	@Override
	public void func_180663_b(World world, BlockPos pos, IBlockState state){
		TileEntity te = world.func_175625_s(pos);
		if (te instanceof IInventory)
		{
			InventoryHelper.func_180175_a(world, pos, (IInventory)te);
			((IInventory)te).func_174888_l();
			world.func_175666_e(pos, this);
		}
		super.func_180663_b(world, pos, state);
	}

	@Override
	public boolean isPowerSink(ConduitType e){
		return true;
	}

	@Override
	public boolean isPowerSource(ConduitType e){
		return false;
	}

	public ConduitType getType() {return Power.ELECTRIC_POWER;}
}
