package cyano.electricadvantage.machines;

import cyano.electricadvantage.init.Power;
import cyano.poweradvantage.api.ConduitType;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.play.server.SPacketCustomSound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.FMLLog;

import java.util.Arrays;
import java.util.List;

public abstract class ElectricMachineTileEntity extends cyano.poweradvantage.api.simple.TileEntitySimplePowerMachine{

	private final ItemStack[] inventory;
	private final int[] inputSlots;
	private final int[] outputSlots;
	private final int[] ioSlots;
	private final int[] otherSlots;
	
	
	
	public ElectricMachineTileEntity(String name,float bufferSize,int numInputSlots, int numOutputSlots, int numOtherSlots) {
		this(name,Power.ELECTRIC_POWER,bufferSize,numInputSlots,numOutputSlots,numOtherSlots);
	}
	
	public ElectricMachineTileEntity(String name, ConduitType powerType, float bufferSize,int numInputSlots, int numOutputSlots, int numOtherSlots) {
		super(powerType, bufferSize, name);
		inventory = new ItemStack[numInputSlots+numOutputSlots+numOtherSlots];
		inputSlots = new int[numInputSlots];
		outputSlots = new int[numOutputSlots];
		ioSlots = new int[numInputSlots+numOutputSlots];
		otherSlots = new int[numOtherSlots];
		for(int i = 0; i < inventory.length; i++){
			if(i < inputSlots.length)inputSlots[i] = i;
			if(i < outputSlots.length)outputSlots[i] = inputSlots.length + i;
			if(i < ioSlots.length)ioSlots[i] = i;
			if(i < otherSlots.length)otherSlots[i] = inputSlots.length + outputSlots.length + i;
		}
	}
	public ElectricMachineTileEntity(String name,int numInputSlots, int numOutputSlots, int numOtherSlots) {
		this(name,500f,numInputSlots,numOutputSlots,numOtherSlots);
	}
	public ElectricMachineTileEntity(String name,int numInputSlots, int numOutputSlots, int numOtherSlots, ConduitType[] energyTypes, float[] energyBuffers) {
		super(energyTypes, energyBuffers, name);
		inventory = new ItemStack[numInputSlots+numOutputSlots+numOtherSlots];
		inputSlots = new int[numInputSlots];
		outputSlots = new int[numOutputSlots];
		ioSlots = new int[numInputSlots+numOutputSlots];
		otherSlots = new int[numOtherSlots];
		for(int i = 0; i < inventory.length; i++){
			if(i < inputSlots.length)inputSlots[i] = i;
			if(i < outputSlots.length)outputSlots[i] = inputSlots.length + i;
			if(i < ioSlots.length)ioSlots[i] = i;
			if(i < otherSlots.length)otherSlots[i] = inputSlots.length + outputSlots.length + i;
		}
	}
	
	private boolean redstone = false;
	private float oldEnergy = 0f;
	private int[] syncArrayOld = null;
	private int[] syncArrayNew = null;
	@Override
	public void powerUpdate(){
		super.powerUpdate();
		
		redstone = func_145831_w().func_175640_z(func_174877_v());
		
		this.setPowerState(this.isPowered());
		
		// automatically detect when a sync is needed
		if(syncArrayOld == null || syncArrayNew == null 
				|| this.getDataFieldArray().length != syncArrayOld.length){
			int size = this.getDataFieldArray().length;
			syncArrayOld = new int[size];
			syncArrayNew = new int[size];
		}
		this.prepareDataFieldsForSync();
		System.arraycopy(this.getDataFieldArray(), 0, syncArrayNew, 0, syncArrayNew.length);
		if(!Arrays.equals(syncArrayOld, syncArrayNew)){
			this.sync();
			System.arraycopy(syncArrayNew, 0, syncArrayOld, 0, syncArrayOld.length);
		}
	}

	
	protected boolean hasRedstoneSignal(){
		return redstone;
	}

	
	protected void setActiveState(boolean active){
		IBlockState oldState = func_145831_w().func_180495_p(func_174877_v());
		if(oldState.func_177230_c() instanceof ElectricMachineBlock 
				&& (Boolean)oldState.func_177229_b(ElectricMachineBlock.ACTIVE) != active ){
			final TileEntity save = this;
			final World w = func_145831_w();
			final BlockPos pos = this.func_174877_v();
			IBlockState newState = oldState.func_177226_a(ElectricMachineBlock.ACTIVE, active);
			w.func_180501_a(pos, newState,3);
			if(save != null){
				w.func_175713_t(pos);
				save.func_145829_t();
				w.func_175690_a(pos, save);
			}
		}
	}
	protected void setPowerState(boolean powered){
		IBlockState oldState = func_145831_w().func_180495_p(func_174877_v());
		if(oldState.func_177230_c() instanceof ElectricMachineBlock 
				&& (Boolean)oldState.func_177229_b(ElectricMachineBlock.POWERED) != powered ){
			final TileEntity save = this;
			final World w = func_145831_w();
			final BlockPos pos = this.func_174877_v();
			IBlockState newState = oldState.func_177226_a(ElectricMachineBlock.POWERED, powered);
			w.func_180501_a(pos, newState, 3);
			if(save != null){
				w.func_175713_t(pos);
				save.func_145829_t();
				w.func_175690_a(pos, save);
			}
		}
	}

	
	@Override
	public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newState)
	{
		// used to allow change in blockstate without interrupting the TileEntity or the GUI
		return (oldState.func_177230_c() != newState.func_177230_c());
	}

	public boolean isActive(){
		return isPowered() && (!this.hasRedstoneSignal() && (this.getEnergy(Power.ELECTRIC_POWER) > 0 || (Boolean)func_145831_w().func_180495_p(func_174877_v()).func_177229_b(ElectricMachineBlock.ACTIVE)));
	}
	
	public abstract boolean isPowered();

	public ItemStack getInputSlot(int i){
		if(i < inputSlots.length){
			return inventory[inputSlots[i]];
		} else {
			FMLLog.warning("Bug in class %s. StackTrace: %s",this.getClass().getName(),Arrays.toString(Thread.currentThread().getStackTrace()).replace(',','\n'));
			return null;
		}
	}
	
	public ItemStack getOutputSlot(int i){
		if(i < outputSlots.length){
			return inventory[outputSlots[i]];
		} else {
			FMLLog.warning("Bug in class %s. StackTrace: %s",this.getClass().getName(),Arrays.toString(Thread.currentThread().getStackTrace()).replace(',','\n'));
			return null;
		}
	}
	
	public ItemStack getOtherSlot(int i){
		if(i < otherSlots.length){
			return inventory[otherSlots[i]];
		} else {
			FMLLog.warning("Bug in class %s. StackTrace: %s",this.getClass().getName(),Arrays.toString(Thread.currentThread().getStackTrace()).replace(',','\n'));
			return null;
		}
	}
	public void setInputSlot(int i, ItemStack item){
		if(i < inputSlots.length){
			inventory[inputSlots[i]] = item;
		} else {
			FMLLog.warning("Bug in class %s. StackTrace: %s",this.getClass().getName(),Arrays.toString(Thread.currentThread().getStackTrace()).replace(',','\n'));
		}
	}
	
	public void setOutputSlot(int i, ItemStack item){
		if(i < outputSlots.length){
			inventory[ outputSlots[i]] = item;
		} else {
			FMLLog.warning("Bug in class %s. StackTrace: %s",this.getClass().getName(),Arrays.toString(Thread.currentThread().getStackTrace()).replace(',','\n'));
		}
	}
	
	public void setOtherSlot(int i, ItemStack item){
		if(i < otherSlots.length){
			inventory[otherSlots[i]] = item;
		} else {
			FMLLog.warning("Bug in class %s. StackTrace: %s",this.getClass().getName(),Arrays.toString(Thread.currentThread().getStackTrace()).replace(',','\n'));
		}
	}
	

	public int numberOfInputSlots(){return inputSlots.length;}
	public int numberOfOutputSlots(){return outputSlots.length;}
	public int numberOfOtherSlots(){return otherSlots.length;}

	
	@Override
	protected ItemStack[] getInventory() {
		return inventory;
	}

	public abstract float[] getProgress();
	
	@Override
	public NBTTagCompound func_189515_b(NBTTagCompound tagRoot){
		super.func_189515_b(tagRoot);
		saveTo(tagRoot);
		return tagRoot;
	}

	@Override
	public void func_145839_a(NBTTagCompound tagRoot){
		super.func_145839_a(tagRoot);
		loadFrom(tagRoot);
	}

	protected abstract void saveTo(NBTTagCompound tagRoot);
	protected abstract void loadFrom(NBTTagCompound tagRoot);

	
	@Override
	public int[] func_180463_a(EnumFacing side) {
		return ioSlots;
	}
	
	@Override
	public boolean func_180461_b(final int slot, final ItemStack targetItem, final EnumFacing side) {
		return isInArray(slot,outputSlots);
	}

	public boolean hasSpaceForItemInOutputSlots(ItemStack stack){
		ItemStack item = stack.func_77946_l();
		for(int slot = 0; slot < numberOfOutputSlots(); slot++){
			ItemStack dest = getOutputSlot(slot);
			if(dest == null) return true;
			int stackLimit = Math.min(dest.func_77976_d(), this.func_70297_j_());
			if(dest.field_77994_a >= stackLimit) continue;
			int combinedStackSize = item.field_77994_a + dest.field_77994_a;
			if(ItemStack.func_179545_c(item, dest) ){
				if(combinedStackSize <= stackLimit){
					return true;
				} else if (item.field_77994_a <= stackLimit) {
					item.field_77994_a -= stackLimit - dest.field_77994_a;
				}
			}
		}
		return false;
	}
	
	public boolean hasSpaceForItemInInputSlots(ItemStack stack){
		ItemStack item = stack.func_77946_l();
		for(int slot = 0; slot < numberOfInputSlots(); slot++){
			ItemStack dest = getInputSlot(slot);
			if(dest == null) return true;
			int stackLimit = Math.min(dest.func_77976_d(), this.func_70297_j_());
			if(dest.field_77994_a >= stackLimit) continue;
			int combinedStackSize = item.field_77994_a + dest.field_77994_a;
			if(ItemStack.func_179545_c(item, dest) ){
				if(combinedStackSize <= stackLimit){
					return true;
				} else if (item.field_77994_a <= stackLimit) {
					item.field_77994_a -= stackLimit - dest.field_77994_a;
				}
			}
		}
		return false;
	}
	
	@Override
	public boolean func_180462_a(final int slot, final ItemStack srcItem, final EnumFacing side) {
		return isValidInputItem(srcItem) && isInArray(slot,inputSlots);
	}

	@Override
	public boolean func_94041_b(final int slot, final ItemStack srcItem) {
		return isValidInputItem(srcItem) && isInArray(slot,inputSlots);
	}
	
	protected abstract boolean isValidInputItem(ItemStack item);
	
	protected final boolean isInArray(int i, int[] arr){
		for(int n = 0; n < arr.length; n++){
			if(arr[n] == i)return true;
		}
		return false;
	}
	
	public ItemStack insertItemToInputSlots(ItemStack itemStack){
        if(itemStack == null) return null;
        if(itemStack.func_77973_b() == null) return null;
		itemStack = itemStack.func_77946_l();
		for(int slot = 0; slot < numberOfInputSlots(); slot++){
			ItemStack slotContent = this.getInputSlot(slot);
			if(slotContent == null){
				// empty slot
				this.setInputSlot(slot,itemStack);
				return null;
			} else if(ItemStack.func_179545_c(itemStack, slotContent)){
				int stackLimit = Math.min(slotContent.func_77976_d(), this.func_70297_j_());
				// found slot with same item
				if(slotContent.field_77994_a < stackLimit){
					// increase stack
					int delta = Math.min(itemStack.field_77994_a, stackLimit - slotContent.field_77994_a);
					slotContent.field_77994_a += delta;
					itemStack.field_77994_a -= delta;
					if(itemStack.field_77994_a <= 0) return null; // done
				}
			}
		}
		// return left-over
		return itemStack;
	}

	protected void playSoundEffect(double x, double y, double z, SoundEvent sound, float volume, float pitch){
		if(func_145831_w().field_72995_K) return;
		final double range = 16;
		List<EntityPlayerMP> players = func_145831_w().func_72872_a(EntityPlayerMP.class, new AxisAlignedBB(
				x - range, y - range, z - range,
				x + range, y + range, z + range));
		for(EntityPlayerMP player : players){
			player.field_71135_a.func_147359_a(new SPacketCustomSound(sound.getRegistryName().toString(), SoundCategory.BLOCKS,
					x, y, z, (float)volume, (float)pitch));
		}
	}
	
	public ItemStack insertItemToOutputSlots(ItemStack itemStack){
        if(itemStack == null) return null;
        if(itemStack.func_77973_b() == null) return null;
		itemStack = itemStack.func_77946_l();
		for(int slot = 0; slot < numberOfOutputSlots(); slot++){
			ItemStack slotContent = this.getOutputSlot(slot);
			if(slotContent == null){
				// empty slot
				this.setOutputSlot(slot,itemStack);
				return null;
			} else if(ItemStack.func_179545_c(itemStack, slotContent)){
				int stackLimit = Math.min(slotContent.func_77976_d(), this.func_70297_j_());
				// found slot with same item
				if(slotContent.field_77994_a < stackLimit){
					// increase stack
					int delta = Math.min(itemStack.field_77994_a, stackLimit - slotContent.field_77994_a);
					slotContent.field_77994_a += delta;
					itemStack.field_77994_a -= delta;
					if(itemStack.field_77994_a <= 0) return null; // done
				}
			}
		}
		// return left-over
		return itemStack;
	}
	
	public int getComparatorOutput(){
		// proportional to number of input slot items
		int sum = 0;
		int total = 0;
		for(int i = 0; i < numberOfInputSlots(); i++){
			ItemStack item = getInputSlot(i);
			if(item == null){
				total += 64;
			} else {
				sum += item.field_77994_a;
				total += item.func_77976_d();
			}
		}
		if(total <= 0) return 0;
		return 15 * sum / total;
	}

	public float getEnergy(){
		return getEnergy(Power.ELECTRIC_POWER);
	}

	public float getEnergyCapacity(){
		return getEnergyCapacity(Power.ELECTRIC_POWER);
	}

	public float addEnergy(float amount){
		return addEnergy(amount,Power.ELECTRIC_POWER);
	}

	public float subtractEnergy(float amount){
		return subtractEnergy(amount,Power.ELECTRIC_POWER);
	}

	public void setEnergy(float amount){ setEnergy(amount,Power.ELECTRIC_POWER); }


	public ConduitType getType() {return Power.ELECTRIC_POWER;}


	@Override public int func_70297_j_(){return 64;}


	@Override
	public boolean isPowerSink(ConduitType e){
		return true;
	}

	@Override
	public boolean isPowerSource(ConduitType e){
		return false;
	}
}
