package cyano.electricadvantage.machines;

import cyano.electricadvantage.init.Blocks;
import cyano.electricadvantage.init.Power;
import cyano.poweradvantage.api.ConduitType;
import cyano.poweradvantage.api.PowerConnectorContext;
import cyano.poweradvantage.conduitnetwork.ConduitRegistry;
import cyano.poweradvantage.init.Fluids;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.Explosion;
import net.minecraft.world.World;

public class ElectricPumpBlock extends ElectricMachineBlock{

	@Override
	public ElectricMachineTileEntity func_149915_a(World w, int m) {
		return new ElectricPumpTileEntity();
	}
	
///// Overrides to make this a multi-type block /////
	
	/**
	 * This method is called whenever the block is placed into the world
	 */
	@Override
	public void func_176213_c(World w, BlockPos coord, IBlockState state){
		super.func_176213_c(w, coord, state);
		ConduitRegistry.getInstance().conduitBlockPlacedEvent(w, w.field_73011_w.getDimension(), coord, cyano.poweradvantage.init.Fluids.fluidConduit_general);
	}
	
	/**
	 * This method is called when the block is removed from the world by an entity.
	 */
	@Override
	public void func_176206_d(World w, BlockPos coord, IBlockState state){
		super.func_176206_d(w, coord, state);
		ConduitRegistry.getInstance().conduitBlockPlacedEvent(w, w.field_73011_w.getDimension(), coord, cyano.poweradvantage.init.Fluids.fluidConduit_general);
		destroyPipe(w,coord);
	}
	/**
	 * This method is called when the block is destroyed by an explosion.
	 */
	@Override
	public void func_180652_a(World w, BlockPos coord, Explosion boom){
		super.func_180652_a(w, coord, boom);
		ConduitRegistry.getInstance().conduitBlockPlacedEvent(w, w.field_73011_w.getDimension(), coord, Fluids.fluidConduit_general);
		destroyPipe(w,coord);
	}
	
	
	private void destroyPipe(World w, BlockPos coord) {
		if(w.field_72995_K) return;
		// destroy connected drill bits
		BlockPos c = coord.func_177977_b();
		while(c.func_177956_o() > 0 && w.func_180495_p(c).func_177230_c() == Blocks.pump_pipe_electric){
			w.func_175698_g(c);
			c = c.func_177977_b();
		}
	}


	@Override
	public boolean canAcceptConnection(PowerConnectorContext connection){
		return  ConduitType.areSameType(getType(), connection.powerType) || ConduitType.areSameType(Fluids.fluidConduit_general, connection.powerType);
	}


	private final ConduitType[] types = {Power.ELECTRIC_POWER, Fluids.fluidConduit_general};

	@Override
	public ConduitType[] getTypes(){
		return types;
	}/**
	 * Determines whether this block/entity should receive energy
	 * @return true if this block/entity should receive energy
	 */
	@Override
	public boolean isPowerSink(ConduitType p){
		return ConduitType.areSameType(Power.ELECTRIC_POWER,p);
	}
	/**
	 * Determines whether this block/entity can provide energy
	 * @return true if this block/entity can provide energy
	 */
	@Override
	public boolean isPowerSource(ConduitType p){
		return Fluids.isFluidType(p);
	}
	
	///// end multi-type overrides /////

}
