package cyano.electricadvantage.machines;

import cyano.electricadvantage.init.Power;
import cyano.poweradvantage.api.ConduitType;
import cyano.poweradvantage.api.PowerConnectorContext;
import cyano.poweradvantage.api.PowerRequest;
import cyano.poweradvantage.conduitnetwork.ConduitRegistry;
import cyano.poweradvantage.init.Fluids;
import net.minecraft.block.Block;
import net.minecraft.block.BlockDynamicLiquid;
import net.minecraft.block.BlockLiquid;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fluids.*;

import java.util.HashSet;
import java.util.Set;

public class ElectricPumpTileEntity extends ElectricMachineTileEntity implements IFluidHandler{


	private final FluidTank tank;
	public static final float ENERGY_COST_PIPE = 100f;
	public static final float ENERGY_COST_VERTICAL = 32f;
	public static final float ENERGY_COST_PUMP = 1000f;
	public static final int VOLUME_PER_PUMP = 1000;
	public static final byte PUMP_INTERVAL = 32;
	public static final byte PIPE_INTERVAL = 11;
	private static final int limit = 5 * 5 * 5; // do not search more than this many blocks at a time



	private final int[] dataSyncArray = new int[4];
	private byte timeUntilNextPump = PUMP_INTERVAL;
	
	

	public ElectricPumpTileEntity() {
		super(ElectricPumpTileEntity.class.getSimpleName(), 0, 0, 0, new ConduitType[]{Power.ELECTRIC_POWER, Fluids.fluidConduit_general},new float[]{ENERGY_COST_PUMP + 256 * ENERGY_COST_VERTICAL,1000f});
		tank = new FluidTank(FluidContainerRegistry.BUCKET_VOLUME);
	}
	

	private int timeToSound = 0;

	@Override
	public void tickUpdate(boolean isServerWorld) {
		if(isServerWorld){
			// server-side logic
			net.minecraft.tileentity.TileEntityPiston k;
			if(!hasRedstoneSignal()){
				if(timeUntilNextPump > 0) timeUntilNextPump--;
				if(timeUntilNextPump == 0 && getTank().getFluidAmount() <= 0){
					World w = func_145831_w();
					boolean success = false;
					BlockPos target = this.func_174877_v().func_177977_b();
					while(target.func_177956_o() > 0 && w.func_180495_p(target).func_177230_c() == cyano.electricadvantage.init.Blocks.pump_pipe_electric){
						target = target.func_177977_b();
					}
					if(target.func_177956_o() > 0 && w.func_175623_d(target) && getEnergy() >= ENERGY_COST_PIPE){
						// place pipe
						w.func_175656_a(target, cyano.electricadvantage.init.Blocks.pump_pipe_electric.func_176223_P());
						this.subtractEnergy(ENERGY_COST_PIPE, Power.ELECTRIC_POWER);
						timeUntilNextPump = PIPE_INTERVAL;
						playSoundEffect(target.func_177958_n()+0.5, target.func_177956_o()+0.5, target.func_177952_p()+0.5, SoundEvents.field_187902_gb, 0.3f, 1f);
					} else {
						// pump fluids
						BlockPos fluidSource = null;
						Set<BlockPos> searchSpace = new HashSet<>(limit + 1); // pre-allocate max memory usage
						fluidSource = scan(searchSpace,target.func_177977_b(), limit);
						if(fluidSource == null) fluidSource = scan(searchSpace,target.func_177978_c(), limit);
						if(fluidSource == null) fluidSource = scan(searchSpace,target.func_177976_e(), limit);
						if(fluidSource == null) fluidSource = scan(searchSpace,target.func_177968_d(), limit);
						if(fluidSource == null) fluidSource = scan(searchSpace,target.func_177974_f(), limit);
						if(fluidSource != null){
							float cost = ENERGY_COST_PUMP+ENERGY_COST_VERTICAL*(func_174877_v().func_177956_o()-fluidSource.func_177956_o());
							// Found a fluid to suck-up!
							IBlockState blockstate = w.func_180495_p(fluidSource);
							Fluid f = getFluid(blockstate);
							if(f != null && getEnergy() >= cost){
								this.getTank().fill(new FluidStack(f,FluidContainerRegistry.BUCKET_VOLUME), true);
								this.subtractEnergy(cost, Power.ELECTRIC_POWER);
								w.func_175698_g(fluidSource);
								success = true;
							}
						}
						timeUntilNextPump = PUMP_INTERVAL;
					}
					if(success){
						playSoundEffect(func_174877_v().func_177958_n()+0.5, func_174877_v().func_177956_o()+0.5, func_174877_v().func_177952_p()+0.5, SoundEvents.field_187630_M, 0.5f, 1f);
						playSoundEffect(func_174877_v().func_177958_n()+0.5, func_174877_v().func_177956_o()+0.5, func_174877_v().func_177952_p()+0.5, SoundEvents.field_187715_dR, 0.3f, 1f);
						timeToSound = 14;
					}
				}
			}
			if(timeToSound == 1){
				playSoundEffect(func_174877_v().func_177958_n()+0.5, func_174877_v().func_177956_o()+0.5, func_174877_v().func_177952_p()+0.5, SoundEvents.field_187712_dQ, 0.3f, 1f);
			}
			if(timeToSound > 0) timeToSound--;
		}
	}
	
	
	@Override
	public boolean isPowered() {
		return getEnergy() > ENERGY_COST_PUMP;
	}



	private final float[] progs = new float[0];
	@Override
	public float[] getProgress() {
		return progs;
	}




	@Override
	protected boolean isValidInputItem(ItemStack item) {
		return false;
	}
	

	private Fluid getFluid(IBlockState blockstate) {
		return FluidRegistry.lookupFluidForBlock(blockstate.func_177230_c());
	}



	private BlockPos scan(Set<BlockPos> searchSpace, BlockPos coord, int limit) {
		if(isFluidBlock(coord) == false) return null;
		do{
			if(isSourceBlock(coord)){
				return coord;
			} else {
				searchSpace.add(coord);
				BlockPos next;
				boolean nomore = true;
				for(EnumFacing dir : EnumFacing.field_82609_l){
					next = coord.func_177972_a(dir);
					if(!searchSpace.contains(next) && isFluidBlock(next)){
						coord = next;
						nomore = false;
						break;
					}
				}
				if(nomore){
					return null;
				}
			}
		} while(searchSpace.size() < limit);
		return null;
	}
	
	private boolean isSourceBlock( BlockPos coord){
		IBlockState bs = func_145831_w().func_180495_p(coord);
		if(bs.func_177230_c() instanceof BlockLiquid){
			return (Integer)bs.func_177229_b(BlockDynamicLiquid.field_176367_b) == 0;
		} else if(bs.func_177230_c() instanceof BlockFluidClassic){
			return ((BlockFluidClassic)bs.func_177230_c()).isSourceBlock(func_145831_w(), coord);
		}else{
			return false;
		}
	}



	private boolean isFluidBlock(BlockPos coord) {
		World w = func_145831_w();
		if(w.func_175667_e(coord) == false) return false;
		Block b = w.func_180495_p(coord).func_177230_c();
		return (b instanceof BlockLiquid || b instanceof IFluidBlock);
	}

	
	private boolean redstone = true;
	private float oldEnergy = 0;
	private int oldFluid = 0;
	@Override
	public void powerUpdate(){
		// deliberately NOT calling super.powerUpdate()
		if(this.getTank().getFluidAmount() > 0){
			ConduitType type = Fluids.fluidToConduitType(getTank().getFluid().getFluid());
			float availableAmount = getTank().getFluidAmount();
			float delta = ConduitRegistry.transmitPowerToConsumers(availableAmount, cyano.poweradvantage.init.Fluids.fluidConduit_general, type, 
					PowerRequest.LAST_PRIORITY, func_145831_w(), func_174877_v(), this);
			if(delta > 0){
				getTank().drain(Math.max((int)delta,1),true); // no free energy!
			}
		}
		// powerUpdate occurs once every 8 world ticks and is scheduled such that neighboring 
		// machines don't powerUpdate in the same world tick. To reduce network congestion, 
		// I'm doing the synchonization logic here instead of in the tickUpdate method
		boolean updateFlag = false;

		if(oldEnergy != getEnergy()){
			oldEnergy = getEnergy();
			updateFlag = true;
		}
		if(oldFluid != getTank().getFluidAmount()){
			oldFluid = getTank().getFluidAmount();
			updateFlag = true;
		}

		redstone = hasRedstoneSignal();
		this.setPowerState(this.isPowered());

		if(updateFlag){
			super.sync();
		}
	}
	@Override
	protected boolean hasRedstoneSignal() {
		return func_145831_w().func_175640_z(func_174877_v());
	}
	
	

	public FluidTank getTank(){
		return tank;
	}

	private final ConduitType[] types = {Power.ELECTRIC_POWER, Fluids.fluidConduit_general};

	@Override
	public ConduitType[] getTypes(){
		return types;
	}

	@Override
	public int[] getDataFieldArray() {
		return dataSyncArray;
	}

	@Override
	public void prepareDataFieldsForSync() {
		dataSyncArray[0] = Float.floatToRawIntBits(this.getEnergy());
		dataSyncArray[1] = this.getTank().getFluidAmount();
		dataSyncArray[2] = (this.getTank().getFluidAmount() > 0 ? FluidRegistry.getFluidID(this.getTank().getFluid().getFluid()) : FluidRegistry.getFluidID(FluidRegistry.WATER));
		dataSyncArray[3] = this.timeUntilNextPump;
	}

	@Override
	public void onDataFieldUpdate() {
		this.setEnergy(Float.intBitsToFloat(dataSyncArray[0]), Power.ELECTRIC_POWER);
		this.getTank().setFluid(new FluidStack(FluidRegistry.getFluid(dataSyncArray[2]),dataSyncArray[1]));
		this.timeUntilNextPump = (byte)dataSyncArray[3];
	}

	

	/**
	 * Handles data saving and loading
	 * @param tagRoot An NBT tag
	 */
	@Override
	public void saveTo(final NBTTagCompound tagRoot) {
		NBTTagCompound tankTag = new NBTTagCompound();
		this.getTank().writeToNBT(tankTag);
		tagRoot.func_74782_a("Tank", tankTag);
		tagRoot.func_74774_a("NextPump", (byte)this.timeUntilNextPump);
	}
	/**
	 * Handles data saving and loading
	 * @param tagRoot An NBT tag
	 */
	@Override
	public void loadFrom(final NBTTagCompound tagRoot) {
		if (tagRoot.func_74764_b("Tank")) {
			NBTTagCompound tankTag = tagRoot.func_74775_l("Tank");
			getTank().readFromNBT(tankTag);
			if(tankTag.func_74764_b("Empty")){
				// empty the tank if NBT says its empty (not default behavior of Tank.readFromNBT(...) )
				getTank().setFluid(null);
			}
		}

		if (tagRoot.func_74764_b("NextPump")) {
			this.timeUntilNextPump = tagRoot.func_74771_c("NextPump");
		}
	}

	@Override
	public int getComparatorOutput() {
		return 15 * getTank().getFluidAmount() / getTank().getCapacity();
	}

///// Overrides to make this a multi-type block /////
	/**
	 * Determines whether this block/entity should receive energy 
	 * @return true if this block/entity should receive energy
	 */
	@Override
	public boolean isPowerSink(ConduitType p){
		return ConduitType.areSameType(Power.ELECTRIC_POWER,p);
	}
	/**
	 * Determines whether this block/entity can provide energy 
	 * @return true if this block/entity can provide energy
	 */
	@Override
	public boolean isPowerSource(ConduitType p){
		return Fluids.isFluidType(p);
	}

	/**
	 * Adds "energy" as a fluid to the FluidTank returned by getTank(). This implementation ignores 
	 * all non-fluid energy types.
	 * @param amount amount of energy/fluid to add
	 * @param type the type of energy/fluid being added.
	 * @return The amount that was actually added
	 */
	@Override
	public float addEnergy(float amount, ConduitType type){
		if(Fluids.isFluidType(type) && type != Fluids.fluidConduit_general){
			if(amount > 0 && this.canFill(null, Fluids.conduitTypeToFluid(type))){
				return this.fill(null, new FluidStack(Fluids.conduitTypeToFluid(type),(int)amount), true);
			} else if (amount < 0 && this.canDrain(null, Fluids.conduitTypeToFluid(type))){
				return -1 * this.drain(null, (int)amount, true).amount;
			}
		}
		return super.addEnergy(amount, type);
	}
	
	/**
	 * Subtracts "energy" as a fluid to the FluidTank returned by getTank(). This implementation 
	 * ignores all non-fluid energy types.
	 * @param amount amount of energy/fluid to add
	 * @param type the type of energy/fluid being added.
	 * @return The amount that was actually added
	 */
	@Override
	public float subtractEnergy(float amount, ConduitType type){
		return addEnergy(-1*amount,type);
	}

	@Override
	public PowerRequest getPowerRequest(ConduitType offer) {
		if(! redstone && ConduitType.areSameType(offer, Power.ELECTRIC_POWER)){
			return new PowerRequest(PowerRequest.LOW_PRIORITY,this.getEnergyCapacity() - this.getEnergy(), this);
		}
		return PowerRequest.REQUEST_NOTHING;
	}


	@Override
	public boolean canAcceptConnection(PowerConnectorContext c){
		return ConduitType.areSameType(getType(), c.powerType) || Fluids.isFluidType( c.powerType);
	}
	
	/**
	 * Implementation of IFluidHandler
	 * @param face Face of the block being polled
	 * @param fluid The fluid being added/removed
	 * @param forReal if true, then the fluid in the tank will change
	 */
	@Override
	public int fill(EnumFacing face, FluidStack fluid, boolean forReal) {
		if(fluid == null) return 0;
		if(getTank().getFluidAmount() <= 0 || getTank().getFluid().getFluid().equals(fluid.getFluid())){
			return getTank().fill(fluid, forReal);
		}
		return 0;
	}
	/**
	 * Implementation of IFluidHandler
	 * @param face Face of the block being polled
	 * @param fluid The fluid being added/removed
	 * @param forReal if true, then the fluid in the tank will change
	 */
	@Override
	public FluidStack drain(EnumFacing face, FluidStack fluid, boolean forReal) {
		if(getTank().getFluidAmount() > 0 && getTank().getFluid().getFluid().equals(fluid.getFluid())){
			return getTank().drain(fluid.amount,forReal);
		} else {
			return getTank().drain(0,false);
		}
	}
	/**
	 * Implementation of IFluidHandler
	 * @param face Face of the block being polled
	 * @param amount The amount of fluid being added/removed
	 * @param forReal if true, then the fluid in the tank will change
	 */
	@Override
	public FluidStack drain(EnumFacing face, int amount, boolean forReal) {
		if(getTank().getFluidAmount() > 0 ){
			return getTank().drain(amount,forReal);
		} else {
			return getTank().drain(0,false);
		}
	}
	/**
	 * Implementation of IFluidHandler
	 * @param face Face of the block being polled
	 * @param fluid The fluid being added/removed
	 */
	@Override
	public boolean canFill(EnumFacing face, Fluid fluid) {
		if(fluid == null) return false;
		if(getTank().getFluidAmount() <= 0) return true;
		return getTank().getFluidAmount() <= getTank().getCapacity() && fluid.equals(getTank().getFluid().getFluid());
	}
	/**
	 * Implementation of IFluidHandler
	 * @param face Face of the block being polled
	 * @param fluid The fluid being added/removed
	 */
	@Override
	public boolean canDrain(EnumFacing face, Fluid fluid) {
		if(fluid == null) return false;
		return getTank().getFluidAmount() > 0 && fluid.equals(getTank().getFluid().getFluid());
	}

	/**
	 * Implementation of IFluidHandler
	 * @param face Face of the block being polled
	 * @return array of FluidTankInfo describing all of the FluidTanks
	 */
	@Override
	public FluidTankInfo[] getTankInfo(EnumFacing face) {
		FluidTankInfo[] arr = new FluidTankInfo[1];
		arr[0] = getTank().getInfo();
		return arr;
	}
	///// end multi-type overrides /////





}
