package cyano.electricadvantage.machines;

import cyano.electricadvantage.init.Power;
import cyano.poweradvantage.api.ConduitType;
import cyano.poweradvantage.api.PowerConnectorContext;
import cyano.poweradvantage.api.PoweredEntity;
import cyano.poweradvantage.conduitnetwork.ConduitRegistry;
import cyano.poweradvantage.init.Fluids;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.Explosion;
import net.minecraft.world.World;


/**
 * The coal boiler is a bit more complicated because it consumes fluids (water) and produces steam, 
 * making it a multi-type powered block.
 * @author DrCyano
 *
 */
public class GrowthChamberControllerBlock extends cyano.poweradvantage.api.simple.BlockSimplePowerMachine{

	
	public GrowthChamberControllerBlock() {
		super(Material.field_76233_E, 0.75f, Power.GROWTHCHAMBER_POWER, Power.ELECTRIC_POWER, Fluids.fluidConduit_general);
	}

	@Override
	public PoweredEntity func_149915_a(World world, int metaDataValue) {
		return new GrowthChamberControllerTileEntity();
	}

	@Override
	public boolean func_149740_M(IBlockState bs) {
		return true;
	}

	@Override
	public int func_180641_l(IBlockState bs, World world, BlockPos coord) {
		if(world.func_175625_s(coord) instanceof GrowthChamberControllerTileEntity){
			return ((GrowthChamberControllerTileEntity)world.func_175625_s(coord)).getComparatorOutput();
		}
		return 0;
	}

	
	
	///// Overrides to make this a multi-type block /////
	
	/**
	 * This method is called whenever the block is placed into the world
	 */
	@Override
	public void func_176213_c(World w, BlockPos coord, IBlockState state){
		super.func_176213_c(w, coord, state);
		ConduitRegistry.getInstance().conduitBlockPlacedEvent(w, w.field_73011_w.getDimension(), coord, cyano.poweradvantage.init.Fluids.fluidConduit_general);
		ConduitRegistry.getInstance().conduitBlockPlacedEvent(w, w.field_73011_w.getDimension(), coord, Power.GROWTHCHAMBER_POWER);
		ConduitRegistry.getInstance().conduitBlockPlacedEvent(w, w.field_73011_w.getDimension(), coord, Power.ELECTRIC_POWER);
	}
	
	/**
	 * This method is called when the block is removed from the world by an entity.
	 */
	@Override
	public void func_176206_d(World w, BlockPos coord, IBlockState state){
		super.func_176206_d(w, coord, state);
		ConduitRegistry.getInstance().conduitBlockRemovedEvent(w, w.field_73011_w.getDimension(), coord, cyano.poweradvantage.init.Fluids.fluidConduit_general);
		ConduitRegistry.getInstance().conduitBlockRemovedEvent(w, w.field_73011_w.getDimension(), coord, Power.GROWTHCHAMBER_POWER);
		ConduitRegistry.getInstance().conduitBlockRemovedEvent(w, w.field_73011_w.getDimension(), coord, Power.ELECTRIC_POWER);
	}
	/**
	 * This method is called when the block is destroyed by an explosion.
	 */
	@Override
	public void func_180652_a(World w, BlockPos coord, Explosion boom){
		super.func_180652_a(w, coord, boom);
		ConduitRegistry.getInstance().conduitBlockRemovedEvent(w, w.field_73011_w.getDimension(), coord, Fluids.fluidConduit_general);
		ConduitRegistry.getInstance().conduitBlockRemovedEvent(w, w.field_73011_w.getDimension(), coord, Power.GROWTHCHAMBER_POWER);
		ConduitRegistry.getInstance().conduitBlockRemovedEvent(w, w.field_73011_w.getDimension(), coord, Power.ELECTRIC_POWER);
	}


	@Override
	public boolean isPowerSink(ConduitType pt){
		return ConduitType.areSameType(Power.ELECTRIC_POWER,pt)
				|| Fluids.isFluidType(pt);
	}
	@Override
	public boolean isPowerSource(ConduitType pt){
		return ConduitType.areSameType(Power.GROWTHCHAMBER_POWER,pt);
	}

	@Override
	public boolean canAcceptConnection(PowerConnectorContext c){
		return ConduitType.areSameType(Power.ELECTRIC_POWER,c.powerType)
				|| ConduitType.areSameType(Power.GROWTHCHAMBER_POWER, c.powerType)
				|| Fluids.isFluidType(c.powerType);
	}

	///// end multi-type overrides /////
}
