package cyano.electricadvantage.machines;

import cyano.electricadvantage.ElectricAdvantage;
import cyano.electricadvantage.init.Power;
import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.scoreboard.Team;
import net.minecraft.util.ITickable;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.*;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.ArrayList;
import java.util.List;

public class LaserTurretTileEntity extends ElectricMachineTileEntity implements ITickable{

	private final SoundEvent LASER_SOUND;

	private static final float RADIANS_TO_DEGREES = (float)(180 / Math.PI);
	private static final float DEGREES_TO_RADIANS = (float)(Math.PI / 180);
	public final static float IDLE_ROTATION_PER_TICK = 2f; // in degrees per tick
	public static final byte LASER_CHARGEUP_TIME = 31;
	private static final byte BLAST_TIME = 10;
	private static final byte HIT_TIME = 7;
	private final static int BURN_TIME = 3;
	public static float ATTACK_DAMAGE = 7f;
	
	public final static float ENERGY_PER_SHOT = 1000;
	public final static float ENERGY_PER_TICK = 1;
	public final static float MAX_BUFFER = ENERGY_PER_SHOT * 2;
	
	private String teamIdentity = null;
	private String playerOwner = null;

	public static final int BLAME_RANGE = 4;
	public static final int TARGET_RANGE = 16;
	public static final int ATTACK_RANGE = 32;
	/** target rotation around Y axis */
	public float rotTargetYaw=0;
	/** target rotation up and down */
	public float rotTargetPitch=0;
	/** rotation around Y axis */
	public float rotYaw=0;
	/** rotation up and down */
	public float rotPitch=0;
	/** previous rotation around Y axis */
	public float rotOldYaw=0;
	/** previous rotation up and down */
	public float rotOldPitch=0;
	/** true when tracking a target, alse otherwise */
	public boolean targetLocked = false;
	/** entityID of target */
	private int targetID = Integer.MIN_VALUE;
	/** laser attack charge-up */
	private int laserAttack = 0;
	/** used by renderer to draw laser line */
	public double laserBlastLength = 0;
	
	private Vec3d opticPosition = null;
	
	/** rotation speed (non-idle), in degrees per tick */
	public final float speed = 9f;


	public LaserTurretTileEntity(){
		super(LaserTurretTileEntity.class.getSimpleName(),MAX_BUFFER,0,0,0);
		SoundEvent soundEvent = SoundEvent.field_187505_a.func_82594_a(new ResourceLocation(ElectricAdvantage.INSTANCE.LASER_SOUND));
		if(soundEvent == null){
			LASER_SOUND = SoundEvents.field_187534_aX;
		} else {
			LASER_SOUND = soundEvent;
		}
	}

	@Override
	public boolean isActive(){
		return  (this.getEnergy() >= ENERGY_PER_SHOT) && (!this.hasRedstoneSignal());
	}
	
	@Override
	public void tickUpdate(boolean b) {
		World w = func_145831_w();
		if(laserAttack > 0){
			laserAttack--;
		}
		boolean isActive = isActive();
		if(w.field_72995_K){
			// client-side only
			if(isActive){
				if(targetLocked == false){
					storeOld();
					rotPitch = 0;
					rotYaw += IDLE_ROTATION_PER_TICK;
					if(rotYaw > 180 ){
						rotYaw -= 360;
						rotOldYaw -= 360;
					}
				} else {
					storeOld();
					Entity e = w.func_73045_a(targetID);
					if(e != null){
						lookAt(e);
					}
					if((rotTargetYaw - rotYaw) > 180){
						rotYaw += 360;
						rotOldYaw += 360;
					} else if((rotTargetYaw - rotYaw) < -180){
						rotYaw -= 360;
						rotOldYaw -= 360;
					}
					rotYaw = clampDelta(rotYaw,rotTargetYaw,speed);
					rotPitch = clampDelta(rotPitch,rotTargetPitch,speed);
					
					// Laser attack animation
					if(laserAttack > 0 && laserAttack <= BLAST_TIME){
						drawLaserLine();
					}
				}
			} else {
				storeOld();
				if(rotPitch > -30){
					rotPitch -= IDLE_ROTATION_PER_TICK;
				}
			}
			
		} else {
			// Server side only
			if(isActive){
				if(targetLocked == false){
					final Block thisBlock = func_145831_w().func_180495_p(func_174877_v()).func_177230_c();
					final boolean isEvil;
					if(thisBlock instanceof LaserTurretBlock
							&& ((LaserTurretBlock)thisBlock).evil){
						isEvil = true;
					} else {
						isEvil = false;
					}
					double x = this.getOpticPosition().field_72450_a;
					double y = this.getOpticPosition().field_72448_b;
					double z = this.getOpticPosition().field_72449_c;
					List<Entity> entities = w.func_72872_a(EntityLivingBase.class, new AxisAlignedBB(x-TARGET_RANGE, y-TARGET_RANGE, z-TARGET_RANGE, x+TARGET_RANGE, y+TARGET_RANGE, z+TARGET_RANGE));
					final double maxRangeSquared = TARGET_RANGE * TARGET_RANGE;
					if(isEvil){
						for (Entity e : entities) {
							if (!(
									e.isCreatureType(EnumCreatureType.MONSTER, false)
									|| e.isCreatureType(EnumCreatureType.AMBIENT, false)
								)
							) {
								if (canSeeEntity(e)) {
									setTarget(e);
									break;
								}
							}
						}
					} else {
						for (Entity e : entities) {
							if (e.func_174791_d().func_72436_e(getOpticPosition()) > maxRangeSquared) continue;
							if (e instanceof EntityPlayer) {
								if (isEnemy((EntityPlayer) e)) {
									setTarget(e);
									break;
								}
							}
							if (e.isCreatureType(EnumCreatureType.MONSTER, false)) {
								if (canSeeEntity(e)) {
									setTarget(e);
									break;
								}
							}
						}
					}
				} 

				validateTarget();
				if(targetLocked ){
					if(laserAttack == HIT_TIME){
						// dish out the laser damage
						playSoundEffect(getOpticPosition().field_72450_a, getOpticPosition().field_72448_b, getOpticPosition().field_72449_c,
								LASER_SOUND,1F,1F);
						List<Entity> victims = getLaserAttackVictims();
						Entity e = w.func_73045_a(targetID);
						for(Entity v : victims){
							hurtVictim(v);
						}
						this.subtractEnergy(ENERGY_PER_SHOT, getType());
						if(e.field_70128_L){
							setTarget(null);
						}
						this.sync();
					} else if(laserAttack == 1){
						laserAttack = LASER_CHARGEUP_TIME;
						this.sync();
					}
				}
				this.subtractEnergy(ENERGY_PER_TICK, getType());
			} else {
				if(targetLocked){
					setTarget(null);
				}
			}

		}
	}
	
	private boolean isEnemy(EntityPlayer player) {
		if(player.field_71075_bZ.field_75098_d) return false;
		if(this.teamIdentity == null || this.teamIdentity.isEmpty()){
			return false;
		} else if(player.func_96124_cp() == null){
			return true;
		} else{
			return !teamIdentity.equals(player.func_96124_cp().func_96661_b());
		}
	}

	
	public boolean showLaserLine(){
		return targetLocked && laserAttack > 0 && laserAttack <= BLAST_TIME;
	}
	
	
	private List<Entity> getLaserAttackVictims(){
		final Vec3d origin = getOpticPosition();
		final Vec3d dir;
		final World w = func_145831_w();
		final Entity e = w.func_73045_a(targetID);
		if(e != null){
			dir = e.func_174791_d().func_72441_c(0, 0.5*e.field_70131_O, 0).func_178788_d(origin).func_72432_b();
		} else {
			dir = (new Vec3d(Math.cos(DEGREES_TO_RADIANS * rotYaw),Math.sin(rotPitch),Math.sin(rotYaw))).func_72432_b();
		}
		final Vec3d terminus = followRayToSolidBlock(origin,dir,ATTACK_RANGE);
		final double maxDistSqr = origin.func_72436_e(terminus);
		final double maxDist = MathHelper.func_76133_a(maxDistSqr);
		List<Entity> potentialVictims = w.func_72872_a(EntityLivingBase.class, new AxisAlignedBB(
				origin.field_72450_a - maxDist, origin.field_72448_b - maxDist, origin.field_72449_c - maxDist,
				origin.field_72450_a + maxDist, origin.field_72448_b + maxDist, origin.field_72449_c + maxDist));
		List<Entity> victims = new ArrayList<>();
		for(Entity v : potentialVictims){
			if(origin.func_72436_e(v.func_174791_d()) < maxDistSqr
					&& rayIntersectsBoundingBox(origin,dir,v.func_174813_aQ())){
				victims.add(v);
			}
		}
		return victims;
	}
	
	

	private void hurtVictim(Entity e){
		e.func_70015_d(BURN_TIME);
		if(e instanceof EntityLivingBase){
			((EntityLivingBase)e).func_70097_a(Power.laser_damage, ATTACK_DAMAGE);
			BlockPos bp = e.func_180425_c();
			if(func_145831_w().func_175623_d(bp) && !func_145831_w().func_175623_d(bp.func_177977_b())){
				func_145831_w().func_175656_a(bp, Blocks.field_150480_ab.func_176223_P());
			}
			if(playerOwner != null && !playerOwner.isEmpty()){
				EntityPlayer p = func_145831_w().func_72924_a(playerOwner);
				if(p != null){
					((EntityLivingBase) e).func_70604_c(p);
				}
			}
		}
	}
	
	private Vec3d calculateLaserTerminus(){
		Vec3d dir;
		World w = func_145831_w();
		Entity e = w.func_73045_a(targetID);
		if(e != null){
			dir = e.func_174791_d().func_72441_c(0, 0.5*e.field_70131_O, 0).func_178788_d(this.getOpticPosition()).func_72432_b();
		} else {
			dir = (new Vec3d(Math.cos(DEGREES_TO_RADIANS * rotYaw),Math.sin(rotPitch),Math.sin(rotYaw))).func_72432_b();
		}
		return followRayToSolidBlock(getOpticPosition(),dir,ATTACK_RANGE);
	}
	private Vec3d followRayToSolidBlock(Vec3d origin, Vec3d dir, double maxRange){
		Vec3d max = origin.func_178787_e(dir).func_72441_c(maxRange * dir.field_72450_a, maxRange * dir.field_72448_b, maxRange * dir.field_72449_c);
		RayTraceResult impact = func_145831_w().func_147447_a(origin, max, true, true, false);
		if(impact != null && impact.field_72313_a == RayTraceResult.Type.BLOCK){
			final Vec3d impactSite;
			if(impact.field_72307_f != null){
				impactSite = impact.field_72307_f;
			} else {
				BlockPos bp = impact.func_178782_a();
				impactSite = new Vec3d(bp.func_177958_n(), bp.func_177956_o(), bp.func_177952_p());
			}
			return impactSite;
		} else {
			return max;
		}
	}
	
	public static boolean rayIntersectsBoundingBox(Vec3d rayOrigin, Vec3d rayDirection, AxisAlignedBB box){
		if(box == null) {
			return false;
		}
		// algorithm from http://gamedev.stackexchange.com/questions/18436/most-efficient-aabb-vs-ray-collision-algorithms
		Vec3d inverse = new Vec3d(1.0 / rayDirection.field_72450_a, 1.0 / rayDirection.field_72448_b, 1.0 / rayDirection.field_72449_c);
		double t1 = (box.field_72340_a - rayOrigin.field_72450_a)*inverse.field_72450_a;
		double t2 = (box.field_72336_d- rayOrigin.field_72450_a)*inverse.field_72450_a;
		double t3 = (box.field_72338_b - rayOrigin.field_72448_b)*inverse.field_72448_b;
		double t4 = (box.field_72337_e - rayOrigin.field_72448_b)*inverse.field_72448_b;
		double t5 = (box.field_72339_c - rayOrigin.field_72449_c)*inverse.field_72449_c;
		double t6 = (box.field_72334_f - rayOrigin.field_72449_c)*inverse.field_72449_c;

		double tmin = max(max(min(t1, t2), min(t3, t4)), min(t5, t6));
		double tmax = min(min(max(t1, t2), max(t3, t4)), max(t5, t6));

		// if tmax < 0, ray (line) is intersecting AABB, but whole AABB is behind us
		if (tmax < 0)
		{
			return false;
		}

		// if tmin > tmax, ray doesn't intersect AABB
		if (tmin > tmax)
		{
			return false;
		}

		return true;
	}

	
	private static Vec3d mul(Vec3d a, double b){
		return new Vec3d(a.field_72450_a * b, a.field_72448_b * b, a.field_72449_c * b);
	}
	private static double max(double a, double b){
		return Math.max(a, b);
	}
	private static double min(double a, double b){
		return Math.min(a, b);
	}
	
	
	private void drawLaserLine() {
		// client-side only
		World w = func_145831_w();
		if(w.field_72995_K){
			this.laserBlastLength = getOpticPosition().func_72438_d(calculateLaserTerminus());
		}
	}

	private void validateTarget() {
		if(targetLocked){
			Entity e = func_145831_w().func_73045_a(targetID);
			if(e == null || e.field_70128_L){
				setTarget(null);
				return;
			}
			
			if(getOpticPosition().func_72438_d(e.func_174791_d()) > TARGET_RANGE){
				setTarget(null);
				return;
			}
			
			if(!canSeeEntity(e)){
				setTarget(null);
				return;
			}
		}
	}
	
	public boolean canSeeEntity(Entity e){
		if(e == null) return false;
		Vec3d offsetOrigin = getOpticPosition().func_178787_e(e.func_174791_d().func_178788_d(getOpticPosition()).func_72432_b());
		RayTraceResult collision = func_145831_w().func_147447_a(offsetOrigin, e.func_174791_d().func_72441_c(0, e.field_70131_O*0.5, 0), true, true, false);
		if(collision != null && collision.field_72313_a == RayTraceResult.Type.BLOCK){
			return false;
		}
		return true;
	}

	public void setTarget(Entity e){
		if(e == null){
			this.targetID = Integer.MIN_VALUE;
			this.targetLocked = false;
			laserAttack = 0;
		} else {
			this.targetID = e.func_145782_y();
			this.targetLocked = true;
			laserAttack = LASER_CHARGEUP_TIME;
			lookAt(e);
		}
		this.sync();
	}

	private void lookAt(Entity e) {
		Vec3d pos = e.func_174791_d().func_72441_c(0, e.field_70131_O*0.5, 0);
		targetPosition(pos);
	}
	
	private Vec3d getOpticPosition(){
		if(opticPosition == null)setOpticPosition();
		return opticPosition;
	}
	
	public void setOwner(EntityPlayer owner){
		this.setTeam(owner.func_96124_cp());
		playerOwner = owner.func_70005_c_();
	}
	
	public void setTeam(Team t){
		if(t == null) {
			teamIdentity = null;
		} else {
			teamIdentity = t.func_96661_b();
		}
	}
	
	@Override 
	public void func_174878_a(BlockPos p){
		super.func_174878_a(p);
		setOpticPosition();
	}
	public final void setOpticPosition(){
		BlockPos p = func_174877_v();
		opticPosition = new Vec3d(p.func_177958_n()+0.5,p.func_177956_o()+0.75,p.func_177952_p()+0.5);
	}
	
	private void targetPosition(Vec3d coord) {
		Vec3d pos = getOpticPosition();
		double x = pos.field_72450_a;
		double y = pos.field_72448_b;
		double z = pos.field_72449_c;
		double dist = distance(x,y,z,coord.field_72450_a, coord.field_72448_b, coord.field_72449_c);
		this.rotTargetYaw = RADIANS_TO_DEGREES * atan2(coord.field_72449_c - z, coord.field_72450_a - x);
		this.rotTargetPitch = RADIANS_TO_DEGREES * asin((coord.field_72448_b - y)/dist);
	}
	
	private double distance(double x1,double y1,double z1,double x2,double y2,double z2){
		double dx = x2-x1;
		double dy = y2-y1;
		double dz = z2-z1;
		return MathHelper.func_76133_a(dx * dx + dy * dy + dz * dz);
	}

	private float atan2(double dy, double dx){
		return (float)Math.atan2(dy, dx);
	}
	
	private float asin(double d){
		return (float)Math.asin(d);
	}

	private static float clampDelta(float old, float goal, float maxDelta){
		float delta = goal - old;
		if(Math.abs(delta) > delta){
			if(delta < 0){
				delta = -1*maxDelta;
			} else {
				delta = maxDelta;
			}
			return old + delta;
		}else{
			return goal;
		}
	}
	
	private void storeOld(){
		rotOldYaw = rotYaw;
		rotOldPitch = rotPitch;
	}
	
	
	
	

	/**
	 * Turns the data field NBT into a network packet
	 */
	@Override 
	public SPacketUpdateTileEntity func_189518_D_(){
		NBTTagCompound nbtTag = new NBTTagCompound();
		nbtTag.func_74776_a("energy", getEnergy());
		nbtTag.func_74757_a("lock", targetLocked);
		if(targetLocked){
			if(targetID != Integer.MIN_VALUE && func_145831_w().func_73045_a(targetID) != null){
				Entity e = func_145831_w().func_73045_a(targetID);
				Vec3d pos = e.func_174791_d().func_72441_c(0, e.field_70131_O*0.5, 0);
				NBTTagCompound target = new NBTTagCompound();
				target.func_74768_a("id", this.targetID);
				target.func_74776_a("yaw", rotTargetYaw);
				target.func_74776_a("pitch", rotTargetPitch);
				target.func_74774_a("attack", (byte)laserAttack);
				nbtTag.func_74782_a("target", target);
			}
		}
		return new SPacketUpdateTileEntity(this.field_174879_c, 0, nbtTag);
	}
	/**
	 * Receives the network packet made by <code>getDescriptionPacket()</code>
	 */
	@Override
	public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity packet) {
		NBTTagCompound tag = packet.func_148857_g();
		if(tag.func_74764_b("lock")){
			this.targetLocked = tag.func_74767_n("lock");
			if(tag.func_74764_b("target")){
				NBTTagCompound target = tag.func_74775_l("target");
				this.targetID = target.func_74762_e("id");
				Entity e = func_145831_w().func_73045_a(targetID);
				if(e != null){
					this.lookAt(e);
				} else {
					this.rotTargetYaw = target.func_74760_g("yaw");
					this.rotTargetPitch = target.func_74760_g("pitch");
				}
				this.laserAttack = target.func_74771_c("attack");
			}
		}
		if(tag.func_74764_b("energy")){
			this.setEnergy(tag.func_74760_g("energy"),getType());
		}
	}

	@Override
	public float[] getProgress() {
		// not used
		return new float[0];
	}

	@Override
	protected void saveTo(NBTTagCompound root) {
		root.func_74776_a("pitch", rotPitch);
		root.func_74776_a("yaw", rotYaw);
		root.func_74757_a("lock", targetLocked);
		if(this.teamIdentity == null){
			root.func_74778_a("team", "");
		} else {
			root.func_74778_a("team", teamIdentity);
		}
		if(this.playerOwner == null){
			root.func_74778_a("player", "");
		} else {
			root.func_74778_a("player", playerOwner);
		}
	}

	@Override
	protected void loadFrom(NBTTagCompound root) {
		if(root.func_74764_b("pitch")){
			this.rotPitch = root.func_74760_g("pitch");
			this.rotOldPitch = rotPitch;
			this.rotTargetPitch = rotPitch;
		}
		if(root.func_74764_b("yaw")){
			this.rotYaw = root.func_74760_g("yaw");
			this.rotOldYaw = rotYaw;
			this.rotTargetYaw = rotYaw;
		}
		if(root.func_74764_b("lock")){
			targetLocked = root.func_74767_n("lock");
		}
		if(root.func_74764_b("team")){
			String teamName = root.func_74779_i("team");
			if(teamName.isEmpty()){
				this.teamIdentity = null;
			}else{
				this.teamIdentity = teamName;
			}
		}

		if(root.func_74764_b("player")){
			playerOwner = root.func_74779_i("player");
		}
	}

	@Override
	protected boolean isValidInputItem(ItemStack item) {
		return false;
	}

	@Override
	public int[] getDataFieldArray() {
		// not used
		return new int[0];
	}

	@Override
	public void onDataFieldUpdate() {
		// not used
	}

	@Override
	public void prepareDataFieldsForSync() {
		// not used
	}

	@Override
	public boolean isPowered() {
		return getEnergy() > ENERGY_PER_TICK;
	}
	
	@Override
	public int getComparatorOutput(){
		if(this.targetLocked){
			return 15;
		} else {
			return (int)(8 * (this.getEnergy() / this.getEnergyCapacity()));
		}
	}
	
	// Helps with laser rendering
	final private int renderRange = ATTACK_RANGE * 2;
	@SideOnly(Side.CLIENT)
	@Override
	public AxisAlignedBB getRenderBoundingBox()
	{
		return new AxisAlignedBB(func_174877_v().func_177982_a(-renderRange, -renderRange, -renderRange), func_174877_v().func_177982_a(renderRange, renderRange, renderRange));
	}
}
